import re

import mdformat


def html_to_markdown(markdown_out: list[str], soup) -> None:
    assert soup is not None, soup
    if isinstance(soup, list):
        for e in soup:
            html_to_markdown(markdown_out, e)
        return

    if soup.name is None:
        markdown_out.append(soup)
    elif soup.name in {'p', 'section', 'div'}:
        markdown_out.append('\n\n')
        html_to_markdown(markdown_out, soup.contents)
        markdown_out.append('\n\n')
    elif soup.name == 'code':
        markdown_out.append('`')
        html_to_markdown(markdown_out, soup.contents)
        markdown_out.append('`')
    elif soup.name == 'a':
        markdown_out.append('[')
        html_to_markdown(markdown_out, soup.contents)
        markdown_out.append('](')
        markdown_out.append(soup['href'])
        markdown_out.append(')')
    elif soup.name == 'b':
        markdown_out.append('**')
        html_to_markdown(markdown_out, soup.contents)
        markdown_out.append('**')
    elif soup.name == 'i':
        markdown_out.append('_')
        html_to_markdown(markdown_out, soup.contents)
        markdown_out.append('_')
    elif soup.name == 'ol':
        for idx, li in enumerate(soup.find_all('li', recursive=False)):
            markdown_out.append(f'{idx + 1}. ')
            html_to_markdown(markdown_out, li.contents)
            del idx, li
    elif soup.name == 'ul':
        for li in soup.find_all('li', recursive=False):
            markdown_out.append('- ')
            html_to_markdown(markdown_out, li.contents)
            del li
    elif soup.name in {'h1', 'h2'}:
        markdown_out.append('## ')
        html_to_markdown(markdown_out, soup.contents)
    elif soup.name == 'h3':
        markdown_out.append('### ')
        html_to_markdown(markdown_out, soup.contents)
    else:
        assert False, soup


MD_FORMAT_OPTIONS = {
    'wrap': 100,
    'number': True,
}


def concat_markdown(parts: list[str]) -> str:
    new_readme = ''.join(parts).strip()
    new_readme = re.sub(r'\n\s*\n', '\n\n', new_readme)
    new_readme = re.sub(r'\n *(\S)', r'\n\1', new_readme)
    return mdformat.text(new_readme, options=MD_FORMAT_OPTIONS)
